@@ -13,11 +13,11 @@ import (
 
 	"github.com/coreos/go-semver/semver"
 	route1client "github.com/openshift/client-go/route/clientset/versioned"
-	extensionsv1beta1 "k8s.io/api/extensions/v1beta1"
+	v1 "k8s.io/api/core/v1"
+	networkingv1 "k8s.io/api/networking/v1"
 	storagev1 "k8s.io/api/storage/v1"
 	apiequality "k8s.io/apimachinery/pkg/api/equality"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/apimachinery/pkg/util/intstr"
 	"k8s.io/client-go/tools/clientcmd"
 
 	cdiv1 "kubevirt.io/containerized-data-importer/pkg/apis/core/v1beta1"
@@ -323,7 +323,7 @@ var _ = Describe("CDI ingress config tests", func() {
 	var (
 		f          = framework.NewFramework("cdiconfig-test")
 		routeStart = func() string { return fmt.Sprintf("%s-%s.", routeName, f.CdiInstallNs) }
-		ingress    *extensionsv1beta1.Ingress
+		ingress    *networkingv1.Ingress
 	)
 
 	BeforeEach(func() {
@@ -366,7 +366,7 @@ var _ = Describe("CDI ingress config tests", func() {
 		Expect(err).ToNot(HaveOccurred())
 		if ingress != nil {
 			By("Cleaning up ingress")
-			err := f.K8sClient.ExtensionsV1beta1().Ingresses(ingress.Namespace).Delete(context.TODO(), ingress.Name, metav1.DeleteOptions{})
+			err := f.K8sClient.NetworkingV1().Ingresses(ingress.Namespace).Delete(context.TODO(), ingress.Name, metav1.DeleteOptions{})
 			Expect(err).ToNot(HaveOccurred())
 			Eventually(func() string {
 				config, err := f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
@@ -382,7 +382,7 @@ var _ = Describe("CDI ingress config tests", func() {
 	It("[test_id:3960]Should set uploadProxyURL if override is not defined", func() {
 		// TODO, don't hard code "cdi-uploadproxy", read it from container env of cdi-deployment deployment.
 		ingress = createIngress("test-ingress", f.CdiInstallNs, "cdi-uploadproxy", ingressUrl)
-		_, err := f.K8sClient.ExtensionsV1beta1().Ingresses(f.CdiInstallNs).Create(context.TODO(), ingress, metav1.CreateOptions{})
+		_, err := f.K8sClient.NetworkingV1().Ingresses(f.CdiInstallNs).Create(context.TODO(), ingress, metav1.CreateOptions{})
 		Expect(err).ToNot(HaveOccurred())
 		By("Expecting uploadproxy url to be " + ingressUrl)
 		Eventually(func() string {
@@ -403,7 +403,7 @@ var _ = Describe("CDI ingress config tests", func() {
 		Expect(err).ToNot(HaveOccurred())
 		// TODO, don't hard code "cdi-uploadproxy", read it from container env of cdi-deployment deployment.
 		ingress = createIngress("test-ingress", f.CdiInstallNs, "cdi-uploadproxy", ingressUrl)
-		_, err = f.K8sClient.ExtensionsV1beta1().Ingresses(f.CdiInstallNs).Create(context.TODO(), ingress, metav1.CreateOptions{})
+		_, err = f.K8sClient.NetworkingV1().Ingresses(f.CdiInstallNs).Create(context.TODO(), ingress, metav1.CreateOptions{})
 		Expect(err).ToNot(HaveOccurred())
 		By("Expecting uploadproxy url to be " + override)
 		Eventually(func() string {
@@ -415,6 +415,23 @@ var _ = Describe("CDI ingress config tests", func() {
 			return *config.Status.UploadProxyURL
 		}, time.Second*30, time.Second).Should(Equal(override))
 	})
+
+	It("Should not crash and uploadProxy empty, if ingress defined without service", func() {
+		ingress = createNoServiceIngress("test-ingress", f.CdiInstallNs)
+		// The cdi controller will not process this ingress because it doesn't have the url we are looking for.
+		_, err := f.K8sClient.NetworkingV1().Ingresses(f.CdiInstallNs).Create(context.TODO(), ingress, metav1.CreateOptions{})
+		Expect(err).ToNot(HaveOccurred())
+		By("Expecting uploadproxy url to be the default url")
+		Eventually(func() string {
+			config, err := f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+			proxyUrl := ""
+			if config.Status.UploadProxyURL != nil {
+				proxyUrl = *config.Status.UploadProxyURL
+			}
+			return proxyUrl
+		}, time.Second*30, time.Second).Should(Equal(defaultUrl))
+	})
 })
 
 var _ = Describe("CDI route config tests", func() {
@@ -475,25 +492,27 @@ var _ = Describe("CDI route config tests", func() {
 var _ = Describe("CDIConfig instance management", func() {
 	f := framework.NewFramework("cdiconfig-test")
 
-	It("[test_id:4952]Should re-create the object if deleted", func() {
-		By("Verifying the object exists")
-		config, err := f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
-		Expect(err).ToNot(HaveOccurred())
-		// Save the UID, so we can check it against a new one.
-		orgUID := config.GetUID()
-		GinkgoWriter.Write([]byte(fmt.Sprintf("Original CDIConfig UID: %s\n", orgUID)))
-		By("Deleting the object")
-		err = f.CdiClient.CdiV1beta1().CDIConfigs().Delete(context.TODO(), config.Name, metav1.DeleteOptions{})
-		Expect(err).ToNot(HaveOccurred())
+	Context("[Destructive]", func() {
+		It("[test_id:4952]Should re-create the object if deleted", func() {
+			By("Verifying the object exists")
+			config, err := f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+			// Save the UID, so we can check it against a new one.
+			orgUID := config.GetUID()
+			GinkgoWriter.Write([]byte(fmt.Sprintf("Original CDIConfig UID: %s\n", orgUID)))
+			By("Deleting the object")
+			err = f.CdiClient.CdiV1beta1().CDIConfigs().Delete(context.TODO(), config.Name, metav1.DeleteOptions{})
+			Expect(err).ToNot(HaveOccurred())
 
-		Eventually(func() bool {
-			newConfig, err := f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
-			if err != nil {
-				return false
-			}
-			GinkgoWriter.Write([]byte(fmt.Sprintf("New CDIConfig UID: %s\n", newConfig.GetUID())))
-			return orgUID != newConfig.GetUID()
-		}, time.Second*30, time.Second).Should(BeTrue())
+			Eventually(func() bool {
+				newConfig, err := f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
+				if err != nil {
+					return false
+				}
+				GinkgoWriter.Write([]byte(fmt.Sprintf("New CDIConfig UID: %s\n", newConfig.GetUID())))
+				return orgUID != newConfig.GetUID() && !apiequality.Semantic.DeepEqual(newConfig.Status, cdiv1.CDIConfigStatus{})
+			}, time.Second*30, time.Second).Should(BeTrue())
+		})
 	})
 })
 
@@ -566,7 +585,7 @@ var _ = Describe("Modifying CDIConfig spec tests", func() {
 			Expect(err).To(HaveOccurred())
 		}
 
-		config, err := f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
+		config, _ := f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
 		if success {
 			By(fmt.Sprintf("CDI Config spec %v", config.Spec.FilesystemOverhead.StorageClass))
 			By(fmt.Sprintf("CDI Config status %v", config.Status.FilesystemOverhead.StorageClass))
@@ -587,25 +606,56 @@ var _ = Describe("Modifying CDIConfig spec tests", func() {
 
 })
 
-func createIngress(name, ns, service, hostUrl string) *extensionsv1beta1.Ingress {
-	return &extensionsv1beta1.Ingress{
+func createIngress(name, ns, service, hostUrl string) *networkingv1.Ingress {
+	return &networkingv1.Ingress{
 		TypeMeta: metav1.TypeMeta{
-			APIVersion: "extensions/v1beta1",
+			APIVersion: "networking/v1",
 		},
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      name,
 			Namespace: ns,
 		},
-		Spec: extensionsv1beta1.IngressSpec{
-			Backend: &extensionsv1beta1.IngressBackend{
-				ServiceName: service,
-				ServicePort: intstr.IntOrString{
-					Type:   intstr.Int,
-					IntVal: 443,
+		Spec: networkingv1.IngressSpec{
+			DefaultBackend: &networkingv1.IngressBackend{
+				Service: &networkingv1.IngressServiceBackend{Name: service, Port: networkingv1.ServiceBackendPort{Number: 443}},
+			},
+			Rules: []networkingv1.IngressRule{
+				{Host: hostUrl},
+			},
+		},
+	}
+}
+
+func createNoServiceIngress(name, ns string) *networkingv1.Ingress {
+	apiGroup := "test.test"
+	return &networkingv1.Ingress{
+		TypeMeta: metav1.TypeMeta{
+			APIVersion: "networking/v1",
+		},
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      name,
+			Namespace: ns,
+			Labels: map[string]string{
+				"nginx.org/ssl-services":                      "cdi-uploadproxy",
+				"ingress.kubernetes.io/ssl-passthrough":       "true",
+				"nginx.ingress.kubernetes.io/secure-backends": "true",
+				"nginx.ingress.kubernetes.io/proxy-body-size": "0",
+			},
+		},
+		Spec: networkingv1.IngressSpec{
+			DefaultBackend: &networkingv1.IngressBackend{
+				Resource: &v1.TypedLocalObjectReference{
+					APIGroup: &apiGroup,
+					Kind:     "test",
+					Name:     "test",
 				},
 			},
-			Rules: []extensionsv1beta1.IngressRule{
-				{Host: hostUrl},
+			TLS: []networkingv1.IngressTLS{
+				{
+					Hosts: []string{
+						"cdi-uploadproxy.example.com",
+					},
+				},
 			},
 		},
 	}
@@ -623,6 +673,7 @@ func SetStorageClassDefault(f *framework.Framework, scName string, isDefault boo
 	ann[controller.AnnDefaultStorageClass] = strconv.FormatBool(isDefault)
 	sc.SetAnnotations(ann)
 	_, err = f.K8sClient.StorageV1().StorageClasses().Update(context.TODO(), sc, metav1.UpdateOptions{})
+	Expect(err).ToNot(HaveOccurred())
 	Eventually(func() string {
 		sc, err := f.K8sClient.StorageV1().StorageClasses().Get(context.TODO(), scName, metav1.GetOptions{})
 		Expect(err).ToNot(HaveOccurred())
