@@ -6,6 +6,7 @@ import (
 	"fmt"
 	"io/ioutil"
 	"net/http"
+	"os"
 	"os/exec"
 	"reflect"
 	"regexp"
@@ -21,7 +22,10 @@ import (
 	v1 "k8s.io/api/core/v1"
 	k8serrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/apimachinery/pkg/labels"
+	"k8s.io/apimachinery/pkg/runtime/schema"
+	"k8s.io/apimachinery/pkg/types"
 
 	cdiv1 "kubevirt.io/containerized-data-importer/pkg/apis/core/v1beta1"
 	"kubevirt.io/containerized-data-importer/pkg/common"
@@ -51,6 +55,7 @@ var _ = Describe("[rfe_id:1115][crit:high][vendor:cnv-qe@redhat.com][level:compo
 	})
 
 	DescribeTable("[test_id:2329] Should fail to import images that require too much space", func(uploadURL string) {
+		startTime := time.Now()
 		imageURL := fmt.Sprintf(uploadURL, f.CdiInstallNs)
 
 		By(imageURL)
@@ -66,21 +71,17 @@ var _ = Describe("[rfe_id:1115][crit:high][vendor:cnv-qe@redhat.com][level:compo
 		Expect(err).NotTo(HaveOccurred(), fmt.Sprintf("Unable to get importer pod"))
 
 		By(fmt.Sprintf("logs for pod -n %s %s", importer.Name, importer.Namespace))
-		By("Verify size error in logs")
+		By("Verify datavolume too small condition")
 		Eventually(func() bool {
-			log, _ := tests.RunKubectlCommand(f, "logs", importer.Name, "-n", importer.Namespace)
-			if strings.Contains(log, "is larger than available size") {
-				return true
-			}
-			if strings.Contains(log, "no space left on device") {
-				return true
-			}
-			if strings.Contains(log, "file largest block is bigger than maxblock") {
-				return true
-			}
-			By("Failed to find error messages about a too large image in log:")
-			By(log)
-			return false
+			dv, err = f.CdiClient.CdiV1beta1().DataVolumes(f.Namespace.Name).Get(context.TODO(), dv.Name, metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+
+			return tests.VerifyConditions(dv.Status.Conditions, startTime, &cdiv1.DataVolumeCondition{
+				Type:    cdiv1.DataVolumeRunning,
+				Status:  v1.ConditionFalse,
+				Message: "DataVolume too small to contain image",
+				Reason:  "Error",
+			})
 		}, controllerSkipPVCCompleteTimeout, assertionPollInterval).Should(BeTrue())
 	},
 		Entry("fail given a large virtual size RAW XZ file", utils.LargeVirtualDiskXz),
@@ -135,6 +136,106 @@ var _ = Describe("[rfe_id:1115][crit:high][vendor:cnv-qe@redhat.com][level:compo
 			Expect(f.GetDiskGroup(f.Namespace, pvc, false)).To(Equal("107"))
 		}
 	})
+
+	It("[test_id:6687] Should retain the importer pod after completion with dv annotation cdi.kubevirt.io/storage.pod.retainAfterCompletion=true", func() {
+		dataVolume := utils.NewDataVolumeWithHTTPImport("import-pod-retain-test", "100Mi", fmt.Sprintf(utils.TinyCoreIsoURL, f.CdiInstallNs))
+		By(fmt.Sprintf("Create new datavolume %s", dataVolume.Name))
+		dataVolume.Annotations[controller.AnnPodRetainAfterCompletion] = "true"
+		dataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dataVolume)
+		Expect(err).ToNot(HaveOccurred())
+
+		By("Verify pvc was created")
+		pvc, err := utils.WaitForPVC(f.K8sClient, dataVolume.Namespace, dataVolume.Name)
+		Expect(err).ToNot(HaveOccurred())
+		f.ForceBindIfWaitForFirstConsumer(pvc)
+
+		By("Wait for import to be completed")
+		err = utils.WaitForDataVolumePhase(f.CdiClient, dataVolume.Namespace, cdiv1.Succeeded, dataVolume.Name)
+		Expect(err).ToNot(HaveOccurred(), "Datavolume not in phase succeeded in time")
+
+		By("Find importer pod after completion")
+		importer, err := utils.FindPodByPrefixOnce(f.K8sClient, dataVolume.Namespace, common.ImporterPodName, common.CDILabelSelector)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(importer.DeletionTimestamp).To(BeNil())
+	})
+})
+
+var _ = Describe("[Istio] Namespace sidecar injection", func() {
+	var (
+		f = framework.NewFramework(namespacePrefix)
+
+		// Istio sidecar injection prevents access to external resources, so we cannot use internal urls (http://cdi-file-host) for the test
+		tinyCoreIsoExternalURL = "http://tinycorelinux.net/12.x/x86/release/TinyCore-current.iso"
+	)
+
+	BeforeEach(func() {
+		value := os.Getenv("KUBEVIRT_DEPLOY_ISTIO")
+		if value != "true" {
+			Skip("No Istio enabled, skipping.")
+		}
+
+		By("Enable istio sidecar injection in namespace")
+		labelPatch := `[{"op":"add","path":"/metadata/labels/istio-injection","value":"enabled" }]`
+		_, err := f.K8sClient.CoreV1().Namespaces().Patch(context.TODO(), f.Namespace.Name, types.JSONPatchType, []byte(labelPatch), metav1.PatchOptions{})
+		Expect(err).ShouldNot(HaveOccurred())
+
+		By("Create istio sidecar")
+		sidecarRes := schema.GroupVersionResource{Group: "networking.istio.io", Version: "v1beta1", Resource: "sidecars"}
+		registryOnlySidecar := generateRegistryOnlySidecar()
+		_, err = f.DynamicClient.Resource(sidecarRes).Namespace(f.Namespace.Name).Create(context.TODO(), registryOnlySidecar, metav1.CreateOptions{})
+		Expect(err).ShouldNot(HaveOccurred())
+	})
+
+	It("[test_id:6498] Should fail to import with namespace sidecar injection enabled, and sidecar.istio.io/inject set to true", func() {
+		dataVolume := utils.NewDataVolumeWithHTTPImport("istio-sidecar-injection-test", "100Mi", tinyCoreIsoExternalURL)
+		By(fmt.Sprintf("Create new datavolume %s", dataVolume.Name))
+		// We set the Immediate Binding annotation to true, to eliminate creation of the consumer pod, which will also fail due to the Istio sidecar.
+		dataVolume.Annotations[controller.AnnImmediateBinding] = "true"
+		dataVolume.Annotations[controller.AnnPodSidecarInjection] = "true"
+		dataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dataVolume)
+		Expect(err).ToNot(HaveOccurred())
+
+		var importer *v1.Pod
+		By("Find importer pod")
+		Eventually(func() bool {
+			importer, err = utils.FindPodByPrefix(f.K8sClient, dataVolume.Namespace, common.ImporterPodName, common.CDILabelSelector)
+			return err == nil
+		}, timeout, pollingInterval).Should(BeTrue())
+
+		By("Verify HTTP request error in importer log")
+		Eventually(func() bool {
+			log, _ := tests.RunKubectlCommand(f, "logs", importer.Name, "-n", importer.Namespace)
+			if strings.Contains(log, "HTTP request errored") {
+				return true
+			}
+			if strings.Contains(log, "502 Bad Gateway") {
+				return true
+			}
+			return false
+		}, time.Minute, pollingInterval).Should(BeTrue())
+	})
+
+	It("[test_id:6492] Should successfully import with namespace sidecar injection enabled and default sidecar.istio.io/inject", func() {
+		dataVolume := utils.NewDataVolumeWithHTTPImport("istio-sidecar-injection-test", "100Mi", tinyCoreIsoExternalURL)
+		By(fmt.Sprintf("Create new datavolume %s", dataVolume.Name))
+		dataVolume.Annotations[controller.AnnImmediateBinding] = "true"
+		dataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dataVolume)
+		Expect(err).ToNot(HaveOccurred())
+
+		By("Verify pvc was created")
+		_, err = utils.WaitForPVC(f.K8sClient, dataVolume.Namespace, dataVolume.Name)
+		Expect(err).ToNot(HaveOccurred())
+
+		By("Find importer pod")
+		Eventually(func() bool {
+			_, err = utils.FindPodByPrefix(f.K8sClient, dataVolume.Namespace, common.ImporterPodName, common.CDILabelSelector)
+			return err == nil
+		}, timeout, pollingInterval).Should(BeTrue())
+
+		By("Wait for import to be completed")
+		err = utils.WaitForDataVolumePhase(f.CdiClient, dataVolume.Namespace, cdiv1.Succeeded, dataVolume.Name)
+		Expect(err).ToNot(HaveOccurred(), "Datavolume not in phase succeeded in time")
+	})
 })
 
 var _ = Describe("[rfe_id:4784][crit:high] Importer respects node placement", func() {
@@ -172,8 +273,8 @@ var _ = Describe("[rfe_id:4784][crit:high] Importer respects node placement", fu
 		Eventually(func() bool {
 			cr, err = f.CdiClient.CdiV1beta1().CDIs().Get(context.TODO(), "cdi", metav1.GetOptions{})
 			Expect(err).ToNot(HaveOccurred())
-			return reflect.DeepEqual(cr.Spec, oldSpec)
-		}, 30*time.Second, time.Second)
+			return reflect.DeepEqual(cr.Spec, *oldSpec)
+		}, 30*time.Second, time.Second).Should(BeTrue())
 	})
 
 	It("[test_id:4783] Should create import pod with node placement", func() {
@@ -185,7 +286,7 @@ var _ = Describe("[rfe_id:4784][crit:high] Importer respects node placement", fu
 			realCR, err := f.CdiClient.CdiV1beta1().CDIs().Get(context.TODO(), "cdi", metav1.GetOptions{})
 			Expect(err).ToNot(HaveOccurred())
 			return reflect.DeepEqual(cr.Spec, realCR.Spec)
-		}, 30*time.Second, time.Second)
+		}, 30*time.Second, time.Second).Should(BeTrue())
 
 		dv := utils.NewDataVolumeWithHTTPImport("node-placement-test", "100Mi", imageTooLargeSize())
 		dv, err = utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dv)
@@ -228,10 +329,10 @@ var _ = Describe("Importer CDI config manipulation tests", func() {
 		})
 
 		Eventually(func() bool {
-			config, err = f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), "cdi", metav1.GetOptions{})
+			config, err = f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
 			Expect(err).ToNot(HaveOccurred())
-			return reflect.DeepEqual(config.Spec, origSpec)
-		}, 30*time.Second, time.Second)
+			return reflect.DeepEqual(config.Spec, *origSpec)
+		}, 30*time.Second, time.Second).Should(BeTrue())
 	})
 
 	dvWithPvcSpec := func() *cdiv1.DataVolume {
@@ -569,7 +670,7 @@ var _ = Describe("Namespace with quota", func() {
 		By(fmt.Sprintf("Creating PVC with endpoint annotation %q", httpEp+"/tinyCore.iso"))
 
 		pvc, err := f.CreatePVCFromDefinition(utils.NewPVCDefinition(
-			"import-image-to-block-pvc",
+			"import-image-to-pvc",
 			"500Mi",
 			pvcAnn,
 			nil))
@@ -585,7 +686,7 @@ var _ = Describe("Namespace with quota", func() {
 		}, CompletionTimeout, assertionPollInterval).Should(BeEquivalentTo(v1.PodSucceeded))
 
 		By("Verify content")
-		same, err := f.VerifyTargetPVCContentMD5(f.Namespace, pvc, "/pvc", utils.TinyCoreBlockMD5, utils.UploadFileSize)
+		same, err := f.VerifyTargetPVCContentMD5(f.Namespace, pvc, utils.DefaultImagePath, utils.UploadFileMD5, utils.UploadFileSize)
 		Expect(err).ToNot(HaveOccurred())
 		Expect(same).To(BeTrue())
 		By("Verifying permissions are 660")
@@ -662,7 +763,7 @@ var _ = Describe("Namespace with quota", func() {
 		}, CompletionTimeout, assertionPollInterval).Should(BeEquivalentTo(v1.PodSucceeded))
 
 		By("Verify content")
-		same, err := f.VerifyTargetPVCContentMD5(f.Namespace, pvc, "/pvc", utils.TinyCoreBlockMD5, utils.UploadFileSize)
+		same, err := f.VerifyTargetPVCContentMD5(f.Namespace, pvc, utils.DefaultImagePath, utils.UploadFileMD5, utils.UploadFileSize)
 		Expect(err).ToNot(HaveOccurred())
 		Expect(same).To(BeTrue())
 		By("Verifying permissions are 660")
@@ -698,7 +799,7 @@ var _ = Describe("Namespace with quota", func() {
 		}, CompletionTimeout, assertionPollInterval).Should(BeEquivalentTo(v1.PodSucceeded))
 
 		By("Verify content")
-		same, err := f.VerifyTargetPVCContentMD5(f.Namespace, pvc, "/pvc", utils.TinyCoreBlockMD5, utils.UploadFileSize)
+		same, err := f.VerifyTargetPVCContentMD5(f.Namespace, pvc, utils.DefaultImagePath, utils.UploadFileMD5, utils.UploadFileSize)
 		Expect(err).ToNot(HaveOccurred())
 		Expect(same).To(BeTrue())
 		By("Verifying permissions are 660")
@@ -959,10 +1060,10 @@ var _ = Describe("Preallocation", func() {
 		})
 
 		Eventually(func() bool {
-			config, err = f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), "cdi", metav1.GetOptions{})
+			config, err = f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
 			Expect(err).ToNot(HaveOccurred())
-			return reflect.DeepEqual(config.Spec, origSpec)
-		}, 30*time.Second, time.Second)
+			return reflect.DeepEqual(config.Spec, *origSpec)
+		}, 30*time.Second, time.Second).Should(BeTrue())
 	})
 
 	It("Importer should add preallocation when requested", func() {
@@ -987,7 +1088,7 @@ var _ = Describe("Preallocation", func() {
 		Expect(pvc.GetAnnotations()[controller.AnnPreallocationApplied]).Should(Equal("true"))
 
 		By("Verify content")
-		md5, err := f.GetMD5(f.Namespace, pvc, "/pvc/disk.img", md5PrefixSize)
+		md5, err := f.GetMD5(f.Namespace, pvc, utils.DefaultImagePath, md5PrefixSize)
 		Expect(err).ToNot(HaveOccurred())
 		Expect(md5).To(Equal(utils.TinyCoreMD5))
 
@@ -1019,7 +1120,7 @@ var _ = Describe("Preallocation", func() {
 		Expect(pvc.GetAnnotations()[controller.AnnPreallocationApplied]).ShouldNot(Equal("true"))
 
 		By("Verify content")
-		md5, err := f.GetMD5(f.Namespace, pvc, "/pvc/disk.img", md5PrefixSize)
+		md5, err := f.GetMD5(f.Namespace, pvc, utils.DefaultImagePath, md5PrefixSize)
 		Expect(err).ToNot(HaveOccurred())
 		Expect(md5).To(Equal(utils.TinyCoreMD5))
 
@@ -1077,40 +1178,40 @@ var _ = Describe("Preallocation", func() {
 			Expect(pvc.GetAnnotations()[controller.AnnPreallocationApplied]).ShouldNot(Equal("true"))
 		}
 	},
-		Entry("HTTP import (ISO image)", true, utils.TinyCoreMD5, "/pvc/disk.img", func() *cdiv1.DataVolume {
+		Entry("HTTP import (ISO image)", true, utils.TinyCoreMD5, utils.DefaultImagePath, func() *cdiv1.DataVolume {
 			return utils.NewDataVolumeWithHTTPImport("import-dv", "100Mi", tinyCoreIsoURL())
 		}),
-		Entry("HTTP import (QCOW2 image)", true, utils.TinyCoreMD5, "/pvc/disk.img", func() *cdiv1.DataVolume {
+		Entry("HTTP import (QCOW2 image)", true, utils.TinyCoreMD5, utils.DefaultImagePath, func() *cdiv1.DataVolume {
 			return utils.NewDataVolumeWithHTTPImport("import-dv", "100Mi", tinyCoreQcow2URL())
 		}),
-		Entry("HTTP import (TAR image)", true, utils.TinyCoreTarMD5, "/pvc/disk.img", func() *cdiv1.DataVolume {
+		Entry("HTTP import (TAR image)", true, utils.TinyCoreTarMD5, utils.DefaultImagePath, func() *cdiv1.DataVolume {
 			return utils.NewDataVolumeWithHTTPImport("import-dv", "100Mi", tinyCoreTarURL())
 		}),
 		Entry("HTTP import (archive content)", false, "", "", func() *cdiv1.DataVolume {
 			return utils.NewDataVolumeWithArchiveContent("import-dv", "100Mi", tinyCoreTarURL())
 		}),
-		Entry("HTTP Import (TAR image, block DataVolume)", true, utils.TinyCoreBlockMD5, "/pvc/disk.img", func() *cdiv1.DataVolume {
+		Entry("HTTP Import (TAR image, block DataVolume)", true, utils.TinyCoreTarMD5, utils.DefaultPvcMountPath, func() *cdiv1.DataVolume {
 			if !f.IsBlockVolumeStorageClassAvailable() {
 				Skip("Storage Class for block volume is not available")
 			}
 
 			return utils.NewDataVolumeWithHTTPImportToBlockPV("import-dv", "4Gi", tinyCoreTarURL(), f.BlockSCName)
 		}),
-		Entry("HTTP Import (ISO image, block DataVolume)", true, utils.TinyCoreBlockMD5, "/pvc/disk.img", func() *cdiv1.DataVolume {
+		Entry("HTTP Import (ISO image, block DataVolume)", true, utils.TinyCoreMD5, utils.DefaultPvcMountPath, func() *cdiv1.DataVolume {
 			if !f.IsBlockVolumeStorageClassAvailable() {
 				Skip("Storage Class for block volume is not available")
 			}
 
 			return utils.NewDataVolumeWithHTTPImportToBlockPV("import-dv", "4Gi", tinyCoreIsoURL(), f.BlockSCName)
 		}),
-		Entry("HTTP Import (QCOW2 image, block DataVolume)", true, utils.TinyCoreBlockMD5, "/pvc/disk.img", func() *cdiv1.DataVolume {
+		Entry("HTTP Import (QCOW2 image, block DataVolume)", true, utils.TinyCoreMD5, utils.DefaultPvcMountPath, func() *cdiv1.DataVolume {
 			if !f.IsBlockVolumeStorageClassAvailable() {
 				Skip("Storage Class for block volume is not available")
 			}
 
 			return utils.NewDataVolumeWithHTTPImportToBlockPV("import-dv", "4Gi", tinyCoreQcow2URL(), f.BlockSCName)
 		}),
-		Entry("ImageIO import", true, utils.ImageioMD5, "/pvc/disk.img", func() *cdiv1.DataVolume {
+		Entry("ImageIO import", true, utils.ImageioMD5, utils.DefaultImagePath, func() *cdiv1.DataVolume {
 			cm, err := utils.CopyImageIOCertConfigMap(f.K8sClient, f.Namespace.Name, f.CdiInstallNs)
 			Expect(err).To(BeNil())
 			stringData := map[string]string{
@@ -1120,14 +1221,14 @@ var _ = Describe("Preallocation", func() {
 			s, _ := utils.CreateSecretFromDefinition(f.K8sClient, utils.NewSecretDefinition(nil, stringData, nil, f.Namespace.Name, "mysecret"))
 			return utils.NewDataVolumeWithImageioImport("import-dv", "100Mi", imageioURL(), s.Name, cm, "123")
 		}),
-		Entry("Registry import", true, utils.TinyCoreMD5, "/pvc/disk.img", func() *cdiv1.DataVolume {
+		Entry("Registry import", true, utils.TinyCoreMD5, utils.DefaultImagePath, func() *cdiv1.DataVolume {
 			dataVolume = utils.NewDataVolumeWithRegistryImport("import-dv", "100Mi", tinyCoreRegistryURL())
 			cm, err := utils.CopyRegistryCertConfigMap(f.K8sClient, f.Namespace.Name, f.CdiInstallNs)
 			Expect(err).To(BeNil())
 			dataVolume.Spec.Source.Registry.CertConfigMap = cm
 			return dataVolume
 		}),
-		Entry("VddkImport", true, utils.VcenterMD5, "/pvc/disk.img", func() *cdiv1.DataVolume {
+		Entry("VddkImport", true, utils.VcenterMD5, utils.DefaultImagePath, func() *cdiv1.DataVolume {
 			// Find vcenter-simulator pod
 			pod, err := utils.FindPodByPrefix(f.K8sClient, f.CdiInstallNs, "vcenter-deployment", "app=vcenter")
 			Expect(err).ToNot(HaveOccurred())
@@ -1157,7 +1258,7 @@ var _ = Describe("Preallocation", func() {
 
 			return utils.NewDataVolumeWithVddkImport("import-dv", "100Mi", backingFile, s.Name, thumbprint, vcenterURL(), vmid.String())
 		}),
-		Entry("Blank image", true, utils.BlankMD5, "/pvc/disk.img", func() *cdiv1.DataVolume {
+		Entry("Blank image", true, utils.BlankMD5, utils.DefaultImagePath, func() *cdiv1.DataVolume {
 			return utils.NewDataVolumeForBlankRawImage("import-dv", "100Mi")
 		}),
 		Entry("Blank block DataVolume", true, utils.BlankMD5, "/pvc", func() *cdiv1.DataVolume {
@@ -1194,7 +1295,7 @@ var _ = Describe("Preallocation", func() {
 		Expect(pvc.GetAnnotations()[controller.AnnPreallocationApplied]).Should(Equal("true"))
 
 		By("Verify content")
-		md5, err := f.GetMD5(f.Namespace, pvc, "/pvc/disk.img", md5PrefixSize)
+		md5, err := f.GetMD5(f.Namespace, pvc, utils.DefaultImagePath, md5PrefixSize)
 		Expect(err).ToNot(HaveOccurred())
 		Expect(md5).To(Equal(utils.BlankMD5))
 
@@ -1203,3 +1304,20 @@ var _ = Describe("Preallocation", func() {
 		Expect(ok).To(BeTrue())
 	})
 })
+
+func generateRegistryOnlySidecar() *unstructured.Unstructured {
+	return &unstructured.Unstructured{
+		Object: map[string]interface{}{
+			"apiVersion": "networking.istio.io/v1beta1",
+			"kind":       "Sidecar",
+			"metadata": map[string]interface{}{
+				"name": "registry-only-sidecar",
+			},
+			"spec": map[string]interface{}{
+				"outboundTrafficPolicy": map[string]interface{}{
+					"mode": "REGISTRY_ONLY",
+				},
+			},
+		},
+	}
+}
